@@ -9,7 +9,7 @@ require Exporter;
 use XSLoader;
 
 our @ISA = qw(Exporter);
-our $VERSION = '0.19';
+our $VERSION = '0.20';
 
 # Items to export into callers namespace by default. Note: do not export
 # names by default without a very good reason. Use EXPORT_OK instead.
@@ -106,37 +106,6 @@ our %EXPORT_TAGS = ( 'all' => [ qw(
 our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 our @EXPORT    = ( @{ $EXPORT_TAGS{'all'} } );
 
-sub AUTOLOAD {
-    # This AUTOLOAD is used to 'autoload' constants from the constant()
-    # XS function.  If a constant is not found then control is passed
-    # to the AUTOLOAD in AutoLoader.
-
-    my $constname;
-    our $AUTOLOAD;
-    ($constname = $AUTOLOAD) =~ s/.*:://;
-    croak "& not defined" if $constname eq 'constant';
-    my $val = constant($constname, @_ ? $_[0] : 0);
-    if ($! != 0) {
-	if ($! =~ /Invalid/ || $!{EINVAL}) {
-	    $AutoLoader::AUTOLOAD = $AUTOLOAD;
-	    goto &AutoLoader::AUTOLOAD;
-	}
-	else {
-	    croak "Your vendor has not defined GSSAPI macro $constname";
-	}
-    }
-    {
-	no strict 'refs';
-	# Fixed between 5.005_53 and 5.005_61
-	if ($] >= 5.00561) {
-	    *$AUTOLOAD = sub () { $val };
-	}
-	else {
-	    *$AUTOLOAD = sub { $val };
-	}
-    }
-    goto &$AUTOLOAD;
-}
 
 XSLoader::load 'GSSAPI', $VERSION;
 
@@ -169,78 +138,6 @@ use constant GSS_C_NO_CREDENTIAL	=> undef;
 use constant GSS_C_NO_CHANNEL_BINDINGS	=> undef;
 use constant GSS_C_EMPTY_BUFFER		=> "";
 
-sub GSS_C_ACCEPT();
-sub GSS_C_AF_APPLETALK();
-sub GSS_C_AF_BSC();
-sub GSS_C_AF_CCITT();
-sub GSS_C_AF_CHAOS();
-sub GSS_C_AF_DATAKIT();
-sub GSS_C_AF_DECnet();
-sub GSS_C_AF_DLI();
-sub GSS_C_AF_DSS();
-sub GSS_C_AF_ECMA();
-sub GSS_C_AF_HYLINK();
-sub GSS_C_AF_IMPLINK();
-sub GSS_C_AF_INET();
-sub GSS_C_AF_LAT();
-sub GSS_C_AF_LOCAL();
-sub GSS_C_AF_NBS();
-sub GSS_C_AF_NS();
-sub GSS_C_AF_NULLADDR();
-sub GSS_C_AF_OSI();
-sub GSS_C_AF_PUP();
-sub GSS_C_AF_SNA();
-sub GSS_C_AF_UNSPEC();
-sub GSS_C_AF_X25();
-sub GSS_C_ANON_FLAG();
-sub GSS_C_BOTH();
-sub GSS_C_CALLING_ERROR_MASK();
-sub GSS_C_CALLING_ERROR_OFFSET();
-sub GSS_C_CONF_FLAG();
-sub GSS_C_DELEG_FLAG();
-sub GSS_C_GSS_CODE();
-sub GSS_C_INDEFINITE();
-sub GSS_C_INITIATE();
-sub GSS_C_INTEG_FLAG();
-sub GSS_C_MECH_CODE();
-sub GSS_C_MUTUAL_FLAG();
-sub GSS_C_PROT_READY_FLAG();
-sub GSS_C_QOP_DEFAULT();
-sub GSS_C_REPLAY_FLAG();
-sub GSS_C_ROUTINE_ERROR_MASK();
-sub GSS_C_ROUTINE_ERROR_OFFSET();
-sub GSS_C_SEQUENCE_FLAG();
-sub GSS_C_SUPPLEMENTARY_MASK();
-sub GSS_C_SUPPLEMENTARY_OFFSET();
-sub GSS_C_TRANS_FLAG();
-sub GSS_S_BAD_BINDINGS();
-sub GSS_S_BAD_MECH();
-sub GSS_S_BAD_NAME();
-sub GSS_S_BAD_NAMETYPE();
-sub GSS_S_BAD_QOP();
-sub GSS_S_BAD_SIG();
-sub GSS_S_BAD_STATUS();
-sub GSS_S_CALL_BAD_STRUCTURE();
-sub GSS_S_CALL_INACCESSIBLE_READ();
-sub GSS_S_CALL_INACCESSIBLE_WRITE();
-sub GSS_S_COMPLETE();
-sub GSS_S_CONTEXT_EXPIRED();
-sub GSS_S_CONTINUE_NEEDED();
-sub GSS_S_CREDENTIALS_EXPIRED();
-sub GSS_S_CRED_UNAVAIL();
-sub GSS_S_DEFECTIVE_CREDENTIAL();
-sub GSS_S_DEFECTIVE_TOKEN();
-sub GSS_S_DUPLICATE_ELEMENT();
-sub GSS_S_DUPLICATE_TOKEN();
-sub GSS_S_FAILURE();
-sub GSS_S_GAP_TOKEN();
-sub GSS_S_NAME_NOT_MN();
-sub GSS_S_NO_CONTEXT();
-sub GSS_S_NO_CRED();
-sub GSS_S_OLD_TOKEN();
-sub GSS_S_UNAUTHORIZED();
-sub GSS_S_UNAVAILABLE();
-sub GSS_S_UNSEQ_TOKEN();
 
 1;
 __END__
@@ -258,37 +155,36 @@ GSSAPI - Perl extension providing access to the GSSAPIv2 library
   my $status;
 
 
-  TRY: {
-     my $target;
-     $status = GSSAPI::Name->import( $target,
-                                     $targethostname,
-                                     GSSAPI::OID::gss_nt_hostbased_service);
-     last if($status->major != GSS_S_COMPLETE );
-     my $tname;
-     $status = $target->display($tname);
-     last if($status->major != GSS_S_COMPLETE );
-     print "\n using Name $tname";
 
+   TRY: {
+       my ($target, $tname, $ttl );
+       $status = GSSAPI::Name->import( $target,
+                                       $server,
+                                       GSSAPI::OID::gss_nt_hostbased_service)
+               or last;
+       $status = $target->display($tname) or last;
+       print "\n using Name $tname";
 
-     my $ctx = GSSAPI::Context->new();
-     my $imech = GSSAPI::OID::gss_mech_krb5;
-     my $iflags = 0 ;
-     my $bindings = GSS_C_NO_CHANNEL_BINDINGS;
-     my $creds = GSS_C_NO_CREDENTIAL;
-     my $itime = 0;
-     my $itoken = '';
-     my ($omech, $otoken, $oflags, $otime);
+       my $ctx = GSSAPI::Context->new();
+       my $imech = GSSAPI::OID::gss_mech_krb5;
+       my $iflags = 0 ;
+       my $bindings = GSS_C_NO_CHANNEL_BINDINGS;
+       my $creds = GSS_C_NO_CREDENTIAL;
+       my $itime = 0;
+       my $itoken = q{};
+       my $otoken;
 
-     $status = $ctx->init($creds,$target,$imech,$iflags,$itime,$bindings,$itoken,
-                          $omech,$otoken,$oflags,$otime);
-  }
+       $status = $ctx->init($creds,$target,
+                            $imech,$iflags,$itime,$bindings,$itoken,
+                            undef, $otoken,undef,undef) or last;
+       $status = $ctx->valid_time_left($ttl) or last;
+       print "\n Security context's time to live $ttl secs";
+   }
 
-  unless ($status->major == GSS_S_COMPLETE ) {
-     print "\nErrors:\n";
-     print $status->generic_message(), "\n", $status->specific_message();
-  } else {
-     print "\n seems everything is fine, type klist to see the ticket\n";
-  }
+   unless ($status->major == GSS_S_COMPLETE  ) {
+      print "\nErrors: ", $status;
+   } else {
+      print "\n seems everything is fine, type klist to see the ticket\n";
 
 
 =head1 DESCRIPTION
@@ -300,8 +196,8 @@ Kerberos-1.2 distribution from MIT.
 Since 0.14 it also compiles and works with Heimdal.
 Lacks of Heimdal support are gss_release_oid(),
 gss_str_to_oid() and fail of some tests.
-Have a look test.pl file too see what tests
-fail on Heimdal (test.pl is just skipping them at the moment)
+Have a look at the tests in t/ directory too see what tests
+fail on Heimdal ( the *.t tests are just skipping them at the moment)
 
 The API presented by this module is a mildly object oriented
 reinterpretation of the C API, where opaque C structures are
@@ -662,6 +558,10 @@ Philip Guenther <pguen@cpan.org>
 
 =item Dax Kelson
 
+=item Marc Lehmann
+
+=item David Leonard
+
 =back
 
 =head1 COPYRIGHT
